package com.mt.wms.empty.task;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mt.wms.core.dal.entity.PointInfo;
import com.mt.wms.core.dal.service.PointInfoServiceBiz;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Author: liguanghao
 * @Date: 2021/12/1 16:56
 * @Version 1.0
 */
@Component
public class TaskDistanceUtils {

    @Autowired
    private PointInfoServiceBiz pointInfoServiceBiz;
    /**
     * 计算各个空闲小车执行此任务的路径长度，传入任务起终点，返回一个小车ID
     * @param startPoint 起点
     * @param endPoint 终点
     * @return 路径最短的小车id
     */
    public  Long chooseVehicle(String startPoint,String endPoint){
        Integer start = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.CODE, startPoint)).getId();
        Integer end = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.CODE, endPoint)).getId();
        //获取点位list
        List<PointInfo> pointInfoList = pointInfoServiceBiz.list(new QueryWrapper<PointInfo>().orderByAsc(PointInfo.ID));
        //计算任务起点到终点的距离
        List<PointInfo> pointInfoListForTask = pointInfoList.subList(start - 1, end - 1);
        Double taskDistance=0D;
        for (PointInfo pointInfo:pointInfoListForTask
             ) {
            taskDistance=taskDistance+pointInfo.getDistance();
        }
        //计算空闲小车当前位置到起点的距离
        // TODO: 2022/2/19  两辆小车都空闲，获取出当前位置
        Integer vehicle1=0;
        Integer vehicle2=39;
        List<PointInfo> pointInfoListForVehicle1 = pointInfoList.subList((vehicle1<start)?vehicle1:start - 1, (end>vehicle1)?end:vehicle1 - 1);
        List<PointInfo> pointInfoListForVehicle2 = pointInfoList.subList((vehicle2<start)?vehicle2:start - 1, (end>vehicle2)?end:vehicle2 - 1);
        //计算得出总距离最短的小车
        Double vehicle1Distance=0D;
        for (PointInfo pointInfo:pointInfoListForVehicle1
        ) {
            vehicle1Distance=vehicle1Distance+pointInfo.getDistance();
        }
        Double vehicle2Distance=0D;
        for (PointInfo pointInfo:pointInfoListForVehicle2
        ) {
            vehicle2Distance=vehicle2Distance+pointInfo.getDistance();
        }
        return (vehicle1Distance< vehicle2Distance)?1L:2L;
    }
    /**
     * 计算执行任务的小车本次任务的剩余路径经过的点位集合
     * @param vehicle 小车id
     * @param startPoint 起点
     * @param endPoint 终点
     * @return 点位集合
     */
    public  List getPointList(Long vehicle,String startPoint,String endPoint){
        Integer start = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.CODE, startPoint)).getId();
        Integer end = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.CODE, endPoint)).getId();
        //获取点位list
        List<PointInfo> pointInfoList = pointInfoServiceBiz.list(new QueryWrapper<PointInfo>().orderByAsc(PointInfo.ID));
        // TODO: 2022/2/19  获取小车当前位置
        Integer nowPoint=3;
        //计算小车当前位置到任务起点-任务起点到任务终点的路径点集合
        //小车当前位置、起点、终点，找出最大最小值
        int min=(((nowPoint<start)?nowPoint:start)<end)?((nowPoint<start)?nowPoint:start):end;
        int max=(((nowPoint>start)?nowPoint:start)>end)?((nowPoint>start)?nowPoint:start):end;
        return pointInfoList.subList(min, max);
    }

    //判断小车是否冲突,传入选定小车的id,任务起终点,另一小车位置。
    public Boolean conflictForVehicle(Long vehicle,String startPoint,String endPoint,Integer otherNowPoint){
        Integer start = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.CODE, startPoint)).getId();
        Integer end = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.CODE, endPoint)).getId();
        //获取点位list
        List<PointInfo> pointInfoList = pointInfoServiceBiz.list(new QueryWrapper<PointInfo>().orderByAsc(PointInfo.ID));
        // TODO: 2022/2/19  获取小车当前位置
        Integer nowPoint=3;
        //计算小车当前位置到任务起点-任务起点到任务终点的路径点集合
        //小车当前位置、起点、终点，找出最大最小值
        int min=(((nowPoint<start)?nowPoint:start)<end)?((nowPoint<start)?nowPoint:start):end;
        int max=(((nowPoint>start)?nowPoint:start)>end)?((nowPoint>start)?nowPoint:start):end;
        if (otherNowPoint>=min&&otherNowPoint<=max){
            return true;
        }else {
            return false;
        }
    }
}
